@@ -55,6 +55,7 @@ type Pager struct {
 	searchString   string
 	searchPattern  *regexp.Regexp
 	gotoLineString string
+	Following      bool
 
 	isShowingHelp bool
 	preHelpState  *_PreHelpState
@@ -73,6 +74,8 @@ type Pager struct {
 	ScrollLeftHint  twin.Cell
 	ScrollRightHint twin.Cell
 
+	SideScrollAmount int // Should be positive
+
 	// If true, pager will clear the screen on return. If false, pager will
 	// clear the last line, and show the cursor.
 	DeInit bool
@@ -82,6 +85,7 @@ type _PreHelpState struct {
 	reader              *Reader
 	scrollPosition      scrollPosition
 	leftColumnZeroBased int
+	following           bool
 }
 
 const _EofMarkerFormat = "\x1b[7m" // Reverse video
@@ -91,17 +95,20 @@ Welcome to Moar, the nice pager!
 
 Miscellaneous
 -------------
-* Press 'q' or ESC to quit
+* Press 'q' or 'ESC' to quit
 * Press 'w' to toggle wrapping of long lines
 * Press '=' to toggle showing the status bar at the bottom
 
 Moving around
 -------------
 * Arrow keys
-* SPACE moves down a page
-* 'g' for going to a specific line number
+* Alt key plus left / right arrow steps one column at a time
 * Left / right can be used to hide / show line numbers
+* CTRL-p moves to the previous line
+* CTRL-n moves to the next line
+* 'g' for going to a specific line number
 * PageUp / 'b' and PageDown / 'f'
+* SPACE moves down a page
 * Home and End for start / end of the document
 * < to go to the start of the document
 * > / 'G' to go to the end of the document
@@ -133,7 +140,11 @@ Source Code
 Available at https://github.com/walles/moar/.
 `)
 
-// NewPager creates a new Pager
+func (pm _PagerMode) isViewing() bool {
+	return pm == _Viewing || pm == _NotFound
+}
+
+// NewPager creates a new Pager with default settings
 func NewPager(r *Reader) *Pager {
 	var name string
 	if r == nil || r.name == nil || len(*r.name) == 0 {
@@ -142,14 +153,15 @@ func NewPager(r *Reader) *Pager {
 		name = "Pager " + *r.name
 	}
 	return &Pager{
-		reader:          r,
-		quit:            false,
-		ShowLineNumbers: true,
-		ShowStatusBar:   true,
-		DeInit:          true,
-		ScrollLeftHint:  twin.NewCell('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
-		ScrollRightHint: twin.NewCell('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
-		scrollPosition:  newScrollPosition(name),
+		reader:           r,
+		quit:             false,
+		ShowLineNumbers:  true,
+		ShowStatusBar:    true,
+		DeInit:           true,
+		SideScrollAmount: 16,
+		ScrollLeftHint:   twin.NewCell('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		ScrollRightHint:  twin.NewCell('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		scrollPosition:   newScrollPosition(name),
 	}
 }
 
@@ -158,7 +170,9 @@ func (p *Pager) setFooter(footer string) {
 
 	pos := 0
 	var footerStyle twin.Style
-	if p.StatusBarStyle == STATUSBAR_STYLE_INVERSE {
+	if standoutStyle != nil {
+		footerStyle = *standoutStyle
+	} else if p.StatusBarStyle == STATUSBAR_STYLE_INVERSE {
 		footerStyle = twin.StyleDefault.WithAttr(twin.AttrReverse)
 	} else if p.StatusBarStyle == STATUSBAR_STYLE_PLAIN {
 		footerStyle = twin.StyleDefault
@@ -189,6 +203,7 @@ func (p *Pager) Quit() {
 	p.reader = p.preHelpState.reader
 	p.scrollPosition = p.preHelpState.scrollPosition
 	p.leftColumnZeroBased = p.preHelpState.leftColumnZeroBased
+	p.Following = p.preHelpState.following
 	p.preHelpState = nil
 }
 
@@ -211,6 +226,14 @@ func (p *Pager) moveRight(delta int) {
 	}
 }
 
+func (p *Pager) handleScrolledUp() {
+	p.Following = false
+}
+
+func (p *Pager) handleScrolledDown() {
+	p.Following = p.isScrolledToEnd()
+}
+
 func (p *Pager) onKey(keyCode twin.KeyCode) {
 	if p.mode == _Searching {
 		p.onSearchKey(keyCode)
@@ -234,30 +257,41 @@ func (p *Pager) onKey(keyCode twin.KeyCode) {
 	case twin.KeyUp:
 		// Clipping is done in _Redraw()
 		p.scrollPosition = p.scrollPosition.PreviousLine(1)
+		p.handleScrolledUp()
 
 	case twin.KeyDown, twin.KeyEnter:
 		// Clipping is done in _Redraw()
 		p.scrollPosition = p.scrollPosition.NextLine(1)
+		p.handleScrolledDown()
 
 	case twin.KeyRight:
-		p.moveRight(16)
+		p.moveRight(p.SideScrollAmount)
 
 	case twin.KeyLeft:
-		p.moveRight(-16)
+		p.moveRight(-p.SideScrollAmount)
+
+	case twin.KeyAltRight:
+		p.moveRight(1)
+
+	case twin.KeyAltLeft:
+		p.moveRight(-1)
 
 	case twin.KeyHome:
 		p.scrollPosition = newScrollPosition("Pager scroll position")
+		p.handleScrolledUp()
 
 	case twin.KeyEnd:
 		p.scrollToEnd()
 
-	case twin.KeyPgDown:
-		_, height := p.screen.Size()
-		p.scrollPosition = p.scrollPosition.NextLine(height - 1)
-
 	case twin.KeyPgUp:
 		_, height := p.screen.Size()
 		p.scrollPosition = p.scrollPosition.PreviousLine(height - 1)
+		p.handleScrolledUp()
+
+	case twin.KeyPgDown:
+		_, height := p.screen.Size()
+		p.scrollPosition = p.scrollPosition.NextLine(height - 1)
+		p.handleScrolledDown()
 
 	default:
 		log.Debugf("Unhandled key event %v", keyCode)
@@ -287,34 +321,43 @@ func (p *Pager) onRune(char rune) {
 				reader:              p.reader,
 				scrollPosition:      p.scrollPosition,
 				leftColumnZeroBased: p.leftColumnZeroBased,
+				following:           p.Following,
 			}
 			p.reader = _HelpReader
 			p.scrollPosition = newScrollPosition("Pager scroll position")
 			p.leftColumnZeroBased = 0
+			p.Following = false
 			p.isShowingHelp = true
 		}
 
 	case '=':
 		p.ShowStatusBar = !p.ShowStatusBar
 
-	case 'k', 'y':
+	// '\x10' = CTRL-p, should scroll up one line.
+	// Ref: https://github.com/walles/moar/issues/107#issuecomment-1328354080
+	case 'k', 'y', '\x10':
 		// Clipping is done in _Redraw()
 		p.scrollPosition = p.scrollPosition.PreviousLine(1)
+		p.handleScrolledUp()
 
-	case 'j', 'e':
+	// '\x0e' = CTRL-n, should scroll down one line.
+	// Ref: https://github.com/walles/moar/issues/107#issuecomment-1328354080
+	case 'j', 'e', '\x0e':
 		// Clipping is done in _Redraw()
 		p.scrollPosition = p.scrollPosition.NextLine(1)
+		p.handleScrolledDown()
 
 	case 'l':
 		// vim right
-		p.moveRight(16)
+		p.moveRight(p.SideScrollAmount)
 
 	case 'h':
 		// vim left
-		p.moveRight(-16)
+		p.moveRight(-p.SideScrollAmount)
 
 	case '<':
 		p.scrollPosition = newScrollPosition("Pager scroll position")
+		p.handleScrolledUp()
 
 	case '>', 'G':
 		p.scrollToEnd()
@@ -322,22 +365,26 @@ func (p *Pager) onRune(char rune) {
 	case 'f', ' ':
 		_, height := p.screen.Size()
 		p.scrollPosition = p.scrollPosition.NextLine(height - 1)
+		p.handleScrolledDown()
 
 	case 'b':
 		_, height := p.screen.Size()
 		p.scrollPosition = p.scrollPosition.PreviousLine(height - 1)
+		p.handleScrolledUp()
 
 	// '\x15' = CTRL-u, should work like just 'u'.
 	// Ref: https://github.com/walles/moar/issues/90
 	case 'u', '\x15':
 		_, height := p.screen.Size()
 		p.scrollPosition = p.scrollPosition.PreviousLine(height / 2)
+		p.handleScrolledUp()
 
 	// '\x04' = CTRL-d, should work like just 'd'.
 	// Ref: https://github.com/walles/moar/issues/90
 	case 'd', '\x04':
 		_, height := p.screen.Size()
 		p.scrollPosition = p.scrollPosition.NextLine(height / 2)
+		p.handleScrolledDown()
 
 	case '/':
 		p.mode = _Searching
@@ -365,7 +412,7 @@ func (p *Pager) onRune(char rune) {
 // StartPaging brings up the pager on screen
 func (p *Pager) StartPaging(screen twin.Screen) {
 	unprintableStyle = p.UnprintableStyle
-	SetManPageFormatFromEnv()
+	ConsumeLessTermcapEnvs()
 
 	p.screen = screen
 
@@ -448,18 +495,19 @@ func (p *Pager) StartPaging(screen twin.Screen) {
 				p.scrollPosition = p.scrollPosition.NextLine(1)
 
 			case twin.MouseWheelLeft:
-				p.moveRight(-16)
+				p.moveRight(-p.SideScrollAmount)
 
 			case twin.MouseWheelRight:
-				p.moveRight(16)
+				p.moveRight(p.SideScrollAmount)
 			}
 
 		case twin.EventResize:
 			// We'll be implicitly redrawn just by taking another lap in the loop
 
 		case eventMoreLinesAvailable:
-			// Doing nothing here is fine; screen will be refreshed on the next
-			// iteration of the main loop.
+			if p.mode.isViewing() && p.Following {
+				p.scrollToEnd()
+			}
 
 		case eventSpinnerUpdate:
 			spinner = event.spinner
