@@ -8,12 +8,16 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/alecthomas/chroma/formatters"
-	"github.com/alecthomas/chroma/styles"
+	"github.com/alecthomas/chroma/v2/formatters"
+	"github.com/alecthomas/chroma/v2/styles"
+	"github.com/google/go-cmp/cmp"
 	"github.com/walles/moar/twin"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 )
 
+const blueBackgroundClearToEol0 = "\x1b[44m\x1b[0K" // With 0 before the K, should clear to EOL
+const blueBackgroundClearToEol = "\x1b[44m\x1b[K"   // No 0 before the K, should also clear to EOL
+
 func TestUnicodeRendering(t *testing.T) {
 	reader := NewReaderFromStream("", strings.NewReader("åäö"))
 
@@ -25,11 +29,11 @@ func TestUnicodeRendering(t *testing.T) {
 
 	contents := startPaging(t, reader).GetRow(0)
 	for pos, expected := range answers {
-		logDifference(t, expected, contents[pos])
+		assertCellsEqual(t, expected, contents[pos])
 	}
 }
 
-func logDifference(t *testing.T, expected twin.Cell, actual twin.Cell) {
+func assertCellsEqual(t *testing.T, expected twin.Cell, actual twin.Cell) {
 	if actual.Rune == expected.Rune && actual.Style == expected.Style {
 		return
 	}
@@ -55,7 +59,7 @@ func TestFgColorRendering(t *testing.T) {
 
 	contents := startPaging(t, reader).GetRow(0)
 	for pos, expected := range answers {
-		logDifference(t, expected, contents[pos])
+		assertCellsEqual(t, expected, contents[pos])
 	}
 }
 
@@ -84,14 +88,14 @@ func TestBrokenUtf8(t *testing.T) {
 
 	contents := startPaging(t, reader).GetRow(0)
 	for pos, expected := range answers {
-		logDifference(t, expected, contents[pos])
+		assertCellsEqual(t, expected, contents[pos])
 	}
 }
 
 func startPaging(t *testing.T, reader *Reader) *twin.FakeScreen {
 	err := reader._wait()
 	if err != nil {
-		panic(err)
+		t.Fatalf("Failed waiting for reader: %v", err)
 	}
 
 	screen := twin.NewFakeScreen(20, 10)
@@ -159,7 +163,7 @@ func TestCodeHighlighting(t *testing.T) {
 		panic("Getting current filename failed")
 	}
 
-	reader, err := NewReaderFromFilename(filename, *styles.Native, formatters.TTY16m)
+	reader, err := NewReaderFromFilename(filename, *styles.Get("native"), formatters.TTY16m)
 	if err != nil {
 		panic(err)
 	}
@@ -180,10 +184,25 @@ func TestCodeHighlighting(t *testing.T) {
 
 	contents := startPaging(t, reader).GetRow(0)
 	for pos, expected := range answers {
-		logDifference(t, expected, contents[pos])
+		assertCellsEqual(t, expected, contents[pos])
 	}
 }
 
+func TestUnicodePrivateUse(t *testing.T) {
+	// This character lives in a Private Use Area:
+	// https://codepoints.net/U+f244
+	//
+	// It's used by Font Awesome as "fa-battery-empty":
+	// https://fontawesome.com/v4/icon/battery-empty
+	char := '\uf244'
+
+	reader := NewReaderFromText("hello", string(char))
+	renderedCell := startPaging(t, reader).GetRow(0)[0]
+
+	// Make sure we display this character unmodified
+	assertCellsEqual(t, twin.NewCell(char, twin.StyleDefault), renderedCell)
+}
+
 func resetManPageFormat() {
 	manPageBold = twin.StyleDefault.WithAttr(twin.AttrBold)
 	manPageUnderline = twin.StyleDefault.WithAttr(twin.AttrUnderline)
@@ -192,7 +211,7 @@ func resetManPageFormat() {
 func testManPageFormatting(t *testing.T, input string, expected twin.Cell) {
 	reader := NewReaderFromStream("", strings.NewReader(input))
 
-	// Without these three lines the man page tests will fail if either of these
+	// Without these lines the man page tests will fail if either of these
 	// environment variables are set when the tests are run.
 	if err := os.Setenv("LESS_TERMCAP_md", ""); err != nil {
 		panic(err)
@@ -200,10 +219,13 @@ func testManPageFormatting(t *testing.T, input string, expected twin.Cell) {
 	if err := os.Setenv("LESS_TERMCAP_us", ""); err != nil {
 		panic(err)
 	}
+	if err := os.Setenv("LESS_TERMCAP_so", ""); err != nil {
+		panic(err)
+	}
 	resetManPageFormat()
 
 	contents := startPaging(t, reader).GetRow(0)
-	logDifference(t, expected, contents[0])
+	assertCellsEqual(t, expected, contents[0])
 	assert.Equal(t, contents[1].Rune, ' ')
 }
 
@@ -331,6 +353,127 @@ func TestScrollToBottomWrapNextToLastLine(t *testing.T) {
 	assert.Equal(t, actual, expected)
 }
 
+// Repro for https://github.com/walles/moar/issues/105
+func TestScrollToEndLongInput(t *testing.T) {
+	const lineCount = 10100 // At least five digits
+
+	// "X" marks the spot
+	reader := NewReaderFromText("test", strings.Repeat(".\n", lineCount-1)+"X")
+	pager := NewPager(reader)
+	pager.ShowLineNumbers = true
+
+	// Tell our Pager to quit immediately
+	pager.Quit()
+
+	// Connect the pager with a screen
+	const screenHeight = 10
+	screen := twin.NewFakeScreen(20, screenHeight)
+	pager.StartPaging(screen)
+
+	// This is what we're really testing
+	pager.scrollToEnd()
+
+	// This makes sure at least one frame gets rendered
+	pager.redraw("")
+
+	// The last screen line holds the status field, and the next to last screen
+	// line holds the last contents line.
+	lastContentsLine := screen.GetRow(screenHeight - 2)
+	firstContentsColumn := len("10_100 ")
+	assertCellsEqual(t, twin.NewCell('X', twin.StyleDefault), lastContentsLine[firstContentsColumn])
+}
+
+func TestIsScrolledToEnd_LongFile(t *testing.T) {
+	// Six lines of contents
+	reader := NewReaderFromText("Testing", "a\nb\nc\nd\ne\nf\n")
+
+	// Three lines screen
+	screen := twin.NewFakeScreen(20, 3)
+
+	// Create the pager
+	pager := NewPager(reader)
+	pager.screen = screen
+
+	assert.Equal(t, false, pager.isScrolledToEnd())
+
+	pager.scrollToEnd()
+	assert.Equal(t, true, pager.isScrolledToEnd())
+}
+
+func TestIsScrolledToEnd_ShortFile(t *testing.T) {
+	// Three lines of contents
+	reader := NewReaderFromText("Testing", "a\nb\nc")
+
+	// Six lines screen
+	screen := twin.NewFakeScreen(20, 6)
+
+	// Create the pager
+	pager := NewPager(reader)
+	pager.screen = screen
+
+	assert.Equal(t, true, pager.isScrolledToEnd())
+
+	pager.scrollToEnd()
+	assert.Equal(t, true, pager.isScrolledToEnd())
+}
+
+func TestIsScrolledToEnd_ExactFile(t *testing.T) {
+	// Three lines of contents
+	reader := NewReaderFromText("Testing", "a\nb\nc")
+
+	// Three lines screen
+	screen := twin.NewFakeScreen(20, 3)
+
+	// Create the pager
+	pager := NewPager(reader)
+	pager.screen = screen
+	pager.ShowStatusBar = false
+
+	assert.Equal(t, true, pager.isScrolledToEnd())
+
+	pager.scrollToEnd()
+	assert.Equal(t, true, pager.isScrolledToEnd())
+}
+
+func TestIsScrolledToEnd_WrappedLastLine(t *testing.T) {
+	// Three lines of contents
+	reader := NewReaderFromText("Testing", "a\nb\nc d e f g h i j k l m n")
+
+	// Three lines screen
+	screen := twin.NewFakeScreen(5, 3)
+
+	// Create the pager
+	pager := NewPager(reader)
+	pager.screen = screen
+	pager.WrapLongLines = true
+
+	assert.Equal(t, false, pager.isScrolledToEnd())
+
+	pager.scrollToEnd()
+	assert.Equal(t, true, pager.isScrolledToEnd())
+
+	pager.onKey(twin.KeyUp)
+	pager.redraw("XXX")
+	assert.Equal(t, false, pager.isScrolledToEnd())
+}
+
+func TestIsScrolledToEnd_EmptyFile(t *testing.T) {
+	// No contents
+	reader := NewReaderFromText("Testing", "")
+
+	// Three lines screen
+	screen := twin.NewFakeScreen(20, 3)
+
+	// Create the pager
+	pager := NewPager(reader)
+	pager.screen = screen
+
+	assert.Equal(t, true, pager.isScrolledToEnd())
+
+	pager.scrollToEnd()
+	assert.Equal(t, true, pager.isScrolledToEnd())
+}
+
 // Verify that we can page all files in ../sample-files/* without crashing
 func TestPageSamples(t *testing.T) {
 	for _, fileName := range getTestFiles() {
@@ -371,6 +514,70 @@ func TestPageSamples(t *testing.T) {
 	}
 }
 
+// Validate rendering of https://en.wikipedia.org/wiki/ANSI_escape_code#EL
+func TestClearToEndOfLine_ClearFromStart(t *testing.T) {
+	screen := startPaging(t, NewReaderFromText("TestClearToEol", blueBackgroundClearToEol))
+
+	screenWidth, _ := screen.Size()
+	var expected []twin.Cell
+	for len(expected) < screenWidth {
+		expected = append(expected,
+			twin.NewCell(' ', twin.StyleDefault.Background(twin.NewColor16(4))),
+		)
+	}
+
+	actual := screen.GetRow(0)
+	assert.DeepEqual(t, actual, expected, cmp.AllowUnexported(twin.Style{}))
+}
+
+// Validate rendering of https://en.wikipedia.org/wiki/ANSI_escape_code#EL
+func TestClearToEndOfLine_ClearFromNotStart(t *testing.T) {
+	screen := startPaging(t, NewReaderFromText("TestClearToEol", "a"+blueBackgroundClearToEol))
+
+	screenWidth, _ := screen.Size()
+	expected := []twin.Cell{
+		twin.NewCell('a', twin.StyleDefault),
+	}
+	for len(expected) < screenWidth {
+		expected = append(expected,
+			twin.NewCell(' ', twin.StyleDefault.Background(twin.NewColor16(4))),
+		)
+	}
+
+	actual := screen.GetRow(0)
+	assert.DeepEqual(t, actual, expected, cmp.AllowUnexported(twin.Style{}))
+}
+
+// Validate rendering of https://en.wikipedia.org/wiki/ANSI_escape_code#EL
+func TestClearToEndOfLine_ClearFromStartScrolledRight(t *testing.T) {
+	pager := NewPager(NewReaderFromText("TestClearToEol", blueBackgroundClearToEol0))
+	pager.ShowLineNumbers = false
+
+	// Tell our Pager to quit immediately
+	pager.Quit()
+
+	// Except for just quitting, this also associates a FakeScreen with the Pager
+	screen := twin.NewFakeScreen(3, 10)
+	pager.StartPaging(screen)
+
+	// Scroll right, this is what we're testing
+	pager.leftColumnZeroBased = 44
+
+	// This makes sure at least one frame gets rendered
+	pager.redraw("")
+
+	screenWidth, _ := screen.Size()
+	var expected []twin.Cell
+	for len(expected) < screenWidth {
+		expected = append(expected,
+			twin.NewCell(' ', twin.StyleDefault.Background(twin.NewColor16(4))),
+		)
+	}
+
+	actual := screen.GetRow(0)
+	assert.DeepEqual(t, actual, expected, cmp.AllowUnexported(twin.Style{}))
+}
+
 func benchmarkSearch(b *testing.B, highlighted bool) {
 	// Pick a go file so we get something with highlighting
 	_, sourceFilename, _, ok := runtime.Caller(0)
@@ -381,7 +588,7 @@ func benchmarkSearch(b *testing.B, highlighted bool) {
 	// Read one copy of the example input
 	var fileContents string
 	if highlighted {
-		highlightedSourceCode, err := highlight(sourceFilename, true, *styles.Native, formatters.TTY16m)
+		highlightedSourceCode, err := highlight(sourceFilename, true, *styles.Get("native"), formatters.TTY16m)
 		if err != nil {
 			panic(err)
 		}
