@@ -1,11 +1,13 @@
 package m
 
 import (
+	"regexp"
 	"strings"
 	"testing"
 
+	"github.com/google/go-cmp/cmp"
 	"github.com/walles/moar/twin"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 )
 
 func testHorizontalCropping(t *testing.T, contents string, firstIndex int, lastIndex int, expected string) {
@@ -61,6 +63,31 @@ func TestEmpty(t *testing.T) {
 	assert.Equal(t, pager.lineNumberOneBased(), 0)
 }
 
+// Repro case for a search bug discovered in v1.9.8.
+func TestSearchHighlight(t *testing.T) {
+	line := Line{
+		raw: "x\"\"x",
+	}
+	pager := Pager{
+		screen:        twin.NewFakeScreen(100, 10),
+		searchPattern: regexp.MustCompile("\""),
+	}
+
+	rendered := pager.renderLine(&line, 1)
+	assert.DeepEqual(t, []renderedLine{
+		{
+			inputLineOneBased: 1,
+			wrapIndex:         0,
+			cells: []twin.Cell{
+				{Rune: 'x', Style: twin.StyleDefault},
+				{Rune: '"', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
+				{Rune: '"', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
+				{Rune: 'x', Style: twin.StyleDefault},
+			},
+		},
+	}, rendered, cmp.AllowUnexported(twin.Style{}), cmp.AllowUnexported(renderedLine{}))
+}
+
 func TestOverflowDown(t *testing.T) {
 	pager := Pager{
 		screen: twin.NewFakeScreen(
